@@ -14,6 +14,8 @@ import copy
 import yfinance as yf
 import warnings
 
+warnings.filterwarnings("ignore")
+
 
 # ------------------------------------------------------------------------------------------
 
@@ -24,7 +26,7 @@ today = dt.date.today()
 class Engine:
 
 
-  def __init__(self,start_date, portfolio, weights=None, rebalance=None, benchmark=['SPY'], end_date=today, optimizer=None, max_vol=0.15, diversification=1,confidences=None, view=None):
+  def __init__(self,start_date, portfolio, weights=None, rebalance=None, benchmark=['SPY'], end_date=today, optimizer=None, max_vol=0.15, diversification=1,confidences=None, view=None, min_weights=None, max_weights=None):
     self.start_date = start_date
     self.end_date = end_date
     self.portfolio = portfolio
@@ -36,6 +38,8 @@ class Engine:
     self.diversification = diversification
     self.view = view
     self.confidences = confidences
+    self.min_weights = min_weights
+    self.max_weights = max_weights
 
     if self.weights==None:
       self.weights = [1.0/len(self.portfolio)]*len(self.portfolio)
@@ -56,7 +60,7 @@ class Engine:
       self.weights = bl(self, perf="False")
       
     if self.rebalance!=None:
-      self.rebalance = make_rebalance(self.start_date, self.end_date, self.optimizer, self.portfolio, self.rebalance)
+      self.rebalance = make_rebalance(self.start_date, self.end_date, self.optimizer, self.portfolio, self.rebalance, self.max_vol, self.weights)
     
 #-------------------------------------------------------------------------------------------
 def get_returns(stocks,wts, start_date, end_date=today):
@@ -392,8 +396,17 @@ def empyrial(my_portfolio, rf=0.0, sigma_value=1, confidence_value=0.95, rebalan
   empyrial.AL = AL
   empyrial.BTA = BTA
 
+  try:
+    empyrial.orderbook = make_rebalance.output
+  except Exception as e:
+    OrderBook = pd.DataFrame(
+        {'Assets': my_portfolio.portfolio,
+        'Allocation': my_portfolio.weights,
+        })
 
-  return qs.plots.returns(returns,benchmark, cumulative=True), qs.plots.monthly_heatmap(returns), qs.plots.drawdown(returns), qs.plots.drawdowns_periods(returns), qs.plots.rolling_volatility(returns), qs.plots.rolling_sharpe(returns), qs.plots.rolling_beta(returns, benchmark)
+    empyrial.orderbook = OrderBook.T
+
+  return qs.plots.returns(returns,benchmark, cumulative=True), qs.plots.monthly_heatmap(returns), qs.plots.drawdown(returns), qs.plots.drawdowns_periods(returns), qs.plots.rolling_volatility(returns), qs.plots.rolling_sharpe(returns), qs.plots.rolling_beta(returns, benchmark), graph_opt(my_portfolio.portfolio, my_portfolio.weights, pie_size=7, font_size=14)
 #---------------------------------------------------------------------------------------------------------------------------------------------------------------
 def flatten(seq):
     l = []
@@ -435,7 +448,11 @@ def efficient_frontier(my_portfolio, perf=True):
 
   #optimize for max sharpe ratio
   ef = EfficientFrontier(mu, S)
-  ef.add_objective(objective_functions.L2_reg, gamma=my_portfolio.diversification)
+
+  if my_portfolio.min_weights != None:
+    ef.add_constraint(lambda x : x >= my_portfolio.min_weights)
+  if my_portfolio.max_weights != None:
+    ef.add_constraint(lambda x : x <= my_portfolio.max_weights)
   weights = ef.max_sharpe()
   cleaned_weights = ef.clean_weights()
   wts = cleaned_weights.items()
@@ -475,7 +492,10 @@ def bl(my_portfolio, perf=True):
   S_bl = bl.bl_cov()
   
   ef = EfficientFrontier(ret_bl, S_bl)
-  ef.add_objective(objective_functions.L2_reg, gamma=my_portfolio.diversification)
+  if my_portfolio.min_weights != None:
+    ef.add_constraint(lambda x : x >= my_portfolio.min_weights)
+  if my_portfolio.max_weights != None:
+    ef.add_constraint(lambda x : x <= my_portfolio.max_weights)
   ef.max_sharpe()
   weights = ef.clean_weights()
   wts = weights.items()
@@ -535,6 +555,10 @@ def mean_var(my_portfolio, vol_max=0.15, perf=True):
 
   ef = EfficientFrontier(mu, S)
   ef.add_objective(objective_functions.L2_reg, gamma=my_portfolio.diversification)
+  if my_portfolio.min_weights != None:
+    ef.add_constraint(lambda x : x >= my_portfolio.min_weights)
+  if my_portfolio.max_weights != None:
+    ef.add_constraint(lambda x : x <= my_portfolio.max_weights)
   ef.efficient_risk(vol_max)
   weights = ef.clean_weights()
 
@@ -562,6 +586,10 @@ def min_var(my_portfolio, perf=True):
 
   ef = EfficientFrontier(mu, S)
   ef.add_objective(objective_functions.L2_reg, gamma=my_portfolio.diversification)
+  if my_portfolio.min_weights != None:
+    ef.add_constraint(lambda x : x >= my_portfolio.min_weights)
+  if my_portfolio.max_weights != None:
+    ef.add_constraint(lambda x : x <= my_portfolio.max_weights)
   ef.min_volatility()
   weights = ef.clean_weights()
 
@@ -801,7 +829,7 @@ def get_date_range(start_date, end_date, rebalance):
     return rebalance_dates
 
 #--------------------------------------------------------------------------------
-def make_rebalance(start_date, end_date, optimizer, portfolio_input, rebalance):
+def make_rebalance(start_date, end_date, optimizer, portfolio_input, rebalance, maxvol,allocation):
     
     print("")
     print("If failed downloads appear they occur because you are querying for a stock that isn't listed for the desired range, no worries we've worked it out.")
@@ -824,15 +852,27 @@ def make_rebalance(start_date, end_date, optimizer, portfolio_input, rebalance):
     #then make a dataframe with the index being the tickers
     output_df = pd.DataFrame(index = portfolio_input)
     
-    #I don't think I need to make a function to make sure the date we have is a trading day because yfinance will fix that 
-    
     for i in range(len(dates) - 1):
+    
+        print(optimizer)
+        type(optimizer)
         
-        portfolio = Engine(
-          start_date = dates[0],
-          end_date = dates[i+1],
-          portfolio = portfolio_input,
-          optimizer = "{}".format(optimizer))
+        if optimizer == "BL":
+          portfolio = Engine(
+            start_date = dates[0],
+            end_date = dates[i+1],
+            weights = allocation,
+            portfolio = portfolio_input)
+          
+
+        else:
+          portfolio = Engine(
+            start_date = dates[0],
+            end_date = dates[i+1],
+            portfolio = portfolio_input,
+            weights = allocation,
+            max_vol = maxvol,
+            optimizer = "{}".format(optimizer))
 
         output_df["{}".format(dates[i+1])] = portfolio.weights
         
@@ -841,8 +881,11 @@ def make_rebalance(start_date, end_date, optimizer, portfolio_input, rebalance):
     portfolio = Engine(
       start_date = dates[0],
       portfolio = portfolio_input,
+      weights = allocation,
+      max_vol = maxvol,
       optimizer = "{}".format(optimizer))
     
     output_df['{}'.format(today)] = portfolio.weights
     
+    make_rebalance.output = output_df
     return output_df
\ No newline at end of file
